from sqlalchemy import Column, Integer, String
from sqlalchemy.orm import sessionmaker
from sqlalchemy.ext.declarative import declarative_base
#
from i18n.translator import Translator

def setup_model(engine):

    Base = declarative_base(bind=engine)
    Session = sessionmaker(bind=engine)

    class TranslationEntry(Base):
        __tablename__ = 'translation_entries'
        language = Column(String, primary_key=True) # destination language, e.g. it_IT
        msgid = Column(String, primary_key=True)
        msgstr = Column(String)
        msgstr_plural = Column(String)

        def __init__(self, language, msgid, msgstr, msgstr_plural=None):
            self.language = language
            self.msgid = msgid
            self.msgstr = msgstr
            self.msgstr_plural = msgstr_plural

        def __repr__(self):
            return "<Translation('%s','%s', '%s')>" % (
                self.language, self.msgid, self.msgstr)

    Base.metadata.create_all(engine)
    return locals()



class DBModel(object):

    def __init__(self, engine):
        self.__dict__.update(setup_model(engine))



class DBTranslator(Translator):

    def __init__(self, rootdir, languages, dest_language=None, autocompile=True,
                 engine=None):
        if engine is None:
            raise TypeError('the engine parameter is non-optional')
        Translator.__init__(self, rootdir, languages, dest_language, autocompile)
        self.engine = engine
        self.db = DBModel(engine)
        self.session = self.db.Session()
        self._cache = {}

    def reload(self):
        Translator.reload(self)
        self.clear_cache()

    def clear_cache(self):
        self._cache.clear()

    def add_translation(self, language, msgid, msgstr, msgstr_plural=None):
        x = self.db.TranslationEntry(language, msgid, msgstr, msgstr_plural)
        self.session.add(x)
        self.session.commit()
        self.clear_cache()

    def _cached(self, fn, *args):
        try:
            return self._cache[args]
        except KeyError:
            res = fn(*args)
            self._cache[args] = res
            return res

    def gettext(self, msgid):
        def get(msgid):
            entry = self._lookup_entry(msgid)
            if entry is not None:
                return entry.msgstr
            else:
                return Translator.gettext(self, msgid)
        #
        return self._cached(get, msgid)

    def ngettext(self, msgid, msgid_plural, n):
        def get(msgid, msgid_plural, is_plural):
            entry = self._lookup_entry(msgid, msgid_plural)
            if entry is not None:
                if is_plural:
                    return entry.msgstr_plural
                else:
                    return entry.msgstr
            else:
                return Translator.ngettext(self, msgid, msgid_plural, n)
        #
        is_plural = (n != 1)
        return self._cached(get, msgid, msgid_plural, is_plural)

    def _lookup_entry(self, msgid, msgid_plural=None):
        Entry = self.db.TranslationEntry
        q = self.session.query(Entry).filter_by(language = self.dest_language,
                                                msgid = msgid)
        if msgid_plural is None:
            q = q.filter(Entry.msgstr_plural == None)
        else:
            q = q.filter(Entry.msgstr_plural != None)
        rows = q.all()
        if len(rows) == 0:
            return None
        elif len(rows) == 1:
            return rows[0]
        else:
            # cannot happen because (language, msgid) is the primary key
            assert False, "Multiple translations of %s found: %s" % (msgid, rows)



import py
import argparse

def get_parser():
    parser = argparse.ArgumentParser(prog='i18n',
                                     description='Manage the translations of a project.')
    parser.add_argument('--root', metavar='DIR', type=py.path.local, default='.',
                        help='The root directory of the project (default: .)')
    parser.add_argument('--languages', metavar='LANG,LANG,...', default='',
                        help='Comma separated list of languages to be supported')
    add_subparsers(parser)
    return parser

def add_subparsers(parser):
    subparsers = parser.add_subparsers(dest='command')
    subparsers.add_parser('extract',
                          help='Extract the messages from the python files ' + 
                          'and merge them with the existing catalogs in languages/')
    subparsers.add_parser('compile', help='Compile the catalogs')


def main(argv):
    from i18n.translator import Translator
    parser = get_parser()
    args = parser.parse_args(argv[1:])
    if args.root.check(dir=False):
        parser.error('Not a directory: %s' % args.root)
    languages = map(str.strip, args.languages.split(','))
    tr = Translator(args.root, languages, autocompile=False)
    if args.command == 'extract':
        tr.extract()
    elif args.command == 'compile':
        tr.compile()
    else:
        parser.error('Invalid command: %s' % args.command)



import py
import os
import gettext
from babel.messages.frontend import CommandLineInterface

babel_cli = CommandLineInterface()

class Translator(object):

    def __init__(self, rootdir, languages, dest_language=None, autocompile=True):
        self.rootdir = py.path.local(rootdir)
        self.languages = languages
        if dest_language is None:
            dest_language = languages[0]
        self.dest_language = dest_language
        self.langdir = self.rootdir.join('languages').ensure(dir=True)
        self.pot = self.langdir.join('template.pot')
        if autocompile:
            self.compile()
        self._init_tr()

    def _init_tr(self):
        self.tr = gettext.translation('messages', str(self.langdir),
                                      [self.dest_language], fallback=True)

    def reload(self):
        self._init_tr()

    def gettext(self, msgid):
        return self.tr.ugettext(msgid)

    def _(self, msgid):
        return self.gettext(msgid)

    def ngettext(self, msgid1, msgid2, n):
        return self.tr.ungettext(msgid1, msgid2, n)

    def _run(self, cmd, *args):
        babel_cli.run(['pybabel', cmd] + list(args))

    def _do_extract(self):
        self._run('extract', '-o', str(self.pot), str(self.rootdir))

    def _do_init(self, lang):
        self._run('init', '-i', str(self.pot), '-l', lang, '-d', str(self.langdir))

    def _do_update(self, lang):
        self._run('update', '-i', str(self.pot), '-l', lang, '-d', str(self.langdir))

    def _do_compile(self, po, lang):
        self._run('compile', '-f', '-i', str(po), '-l', lang, '-d', str(self.langdir))

    def get_po(self, lang):
        return self.langdir.join(lang, 'LC_MESSAGES', 'messages.po')

    def extract(self):
        self._do_extract()
        for lang in self.languages:
            dst = self.get_po(lang)
            if dst.check(file=False):
                self._do_init(lang)
            else:
                self._do_update(lang)

    def compile(self):
        for po in self.langdir.visit('*.po', rec=True):
            mo = po.new(ext='mo')
            if mo.check(exists=True) and mo.mtime() >= po.mtime():
                # the .mo is up to date
                continue
            lang = po.dirpath('..').basename
            self.langdir.join(lang, 'LC_MESSAGES').ensure(dir=1)
            print 'compiling language %s' % po.basename
            self._do_compile(po, lang)

    def cmdline(self, argv):
        import argparse
        from i18n.cmdline import add_subparsers
        parser = argparse.ArgumentParser(
            description='Manage the translations of a project.')
        add_subparsers(parser)
        args = parser.parse_args(argv[1:])
        if args.command == 'extract':
            self.extract()
        elif args.command == 'compile':
            self.compile()
        else:
            parser.error('Invalid command: %s' % args.command)



import sys
import os.path
from setuptools import setup, find_packages

readme = os.path.join(os.path.dirname(__file__), 'README.rst')
long_description = open(readme).read()

setup(
    name='i18n',
    version='0.1',
    author='Antonio Cuni',
    author_email='anto.cuni@gmail.com',
    packages=['i18n'],
    url='http://bitbucket.org/antocuni/i18n',
    license='BSD',
    description='Simplify the development of internationalized applications',
    long_description=long_description,
    keywords='i18n gettext pybabel translation',
    classifiers=[
        "Development Status :: 4 - Beta",
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Internationalization",
        ],
    install_requires=['py', 'babel']
)
